import PvRecorderStatus from "./pv_recorder_status_t";
declare function pvRecorderStatusToException(status: PvRecorderStatus, errorMessage: string): Error;
export default pvRecorderStatusToException;
//# sourceMappingURL=errors.d.ts.map