// adapted from sdk\cs\src\FoundryModelInfo.cs
export var DeviceType;
(function (DeviceType) {
    DeviceType["Invalid"] = "Invalid";
    DeviceType["CPU"] = "CPU";
    DeviceType["GPU"] = "GPU";
    DeviceType["NPU"] = "NPU";
})(DeviceType || (DeviceType = {}));
