/**
 * The SDK protocol version.
 * This must match the version expected by the copilot-agent-runtime server.
 */
export declare const SDK_PROTOCOL_VERSION = 3;
/**
 * Gets the SDK protocol version.
 * @returns The protocol version number
 */
export declare function getSdkProtocolVersion(): number;
